@@ -5,7 +5,10 @@ Runner modules for executing experiments and agents.
 from __future__ import annotations
 
 import asyncio
+import contextvars
 import json
+import inspect
+import os
 import time
 from datetime import datetime
 from pathlib import Path
@@ -38,6 +41,9 @@ class ExperimentRunner:
         self.config = config
         self.no_collector = no_collector
         
+        # Apply project environment (.env + inline environment variables)
+        self._apply_environment()
+
         # Configure output directories (respect config location for relative paths)
         output_base = Path(config.output_directory)
         if not output_base.is_absolute():
@@ -50,8 +56,18 @@ class ExperimentRunner:
         output_base.mkdir(parents=True, exist_ok=True)
 
         offline_dir = output_base / "artifacts"
+        offline_dir.mkdir(parents=True, exist_ok=True)
+        # Ensure downstream load_env() calls don't re-enable collector unintentionally
+        should_use_collector = (not no_collector) and bool(config.collector_url)
+        if not should_use_collector:
+            os.environ["FLUXLOOP_USE_COLLECTOR"] = "false"
+        else:
+            os.environ["FLUXLOOP_USE_COLLECTOR"] = "true"
+        # Pin offline store dir in env so refresh_config respects our path
+        os.environ.setdefault("FLUXLOOP_OFFLINE_DIR", str(offline_dir))
+        os.environ.setdefault("FLUXLOOP_OFFLINE_ENABLED", "true")
         fluxloop.configure(
-            use_collector=not no_collector and bool(config.collector_url),
+            use_collector=should_use_collector,
             collector_url=config.collector_url or None,
             api_key=config.collector_api_key,
             offline_store_enabled=True,
@@ -77,6 +93,31 @@ class ExperimentRunner:
         # Helpers for target loading and argument binding
         self._arg_binder = ArgBinder(config)
     
+    def _apply_environment(self) -> None:
+        """Load environment variables from .env and runner settings."""
+
+        source_dir = self.config.get_source_dir()
+        env_candidates: List[Path] = []
+
+        if source_dir:
+            env_candidates.append(source_dir / ".env")
+            parent = source_dir.parent
+            if parent != source_dir:
+                env_candidates.append(parent / ".env")
+
+        for candidate in env_candidates:
+            if candidate.exists():
+                try:
+                    fluxloop.load_env(candidate, override=True, refresh_config=True)
+                except Exception:
+                    console.log(f"[yellow]Warning:[/yellow] Failed to load environment from {candidate}")
+                else:
+                    break
+
+        env_vars = getattr(self.config.runner, "environment_vars", {}) or {}
+        for key, value in env_vars.items():
+            os.environ[key] = str(value)
+
     def _load_agent(self) -> Callable:
         """Load the agent function from module path."""
         loader = TargetLoader(self.config.runner, source_dir=self.config.get_source_dir())
@@ -315,6 +356,11 @@ class ExperimentRunner:
             final_output = self._extract_final_output(callback_messages, observations)
             if final_output is not None:
                 result = final_output
+            else:
+                # As a fallback, coerce async streams or objects into text
+                coerced = await self._coerce_result_to_text(result)
+                if coerced is not None:
+                    result = coerced
 
             # Mark successful
             self.results["successful"] += 1
@@ -397,11 +443,26 @@ class ExperimentRunner:
             if callable(error_cb) and hasattr(error_cb, "errors"):
                 callback_store["error"] = error_cb.errors
 
-        if asyncio.iscoroutinefunction(agent_func):
-            return await agent_func(**kwargs)
+        if inspect.isasyncgenfunction(agent_func):
+            return await self._consume_async_gen(agent_func, kwargs)
 
-        loop = asyncio.get_event_loop()
-        return await loop.run_in_executor(None, lambda: agent_func(**kwargs))
+        if asyncio.iscoroutinefunction(agent_func):
+            # Ensure current contextvars are preserved for the coroutine
+            ctx = contextvars.copy_context()
+            result = await ctx.run(lambda: agent_func(**kwargs))
+        else:
+            loop = asyncio.get_event_loop()
+            # Preserve contextvars across thread execution
+            ctx = contextvars.copy_context()
+            def _call():
+                return agent_func(**kwargs)
+            result = await loop.run_in_executor(None, lambda: ctx.run(_call))
+
+        # If an async generator/iterable is returned, consume it into a string
+        if inspect.isasyncgen(result) or hasattr(result, "__aiter__"):
+            return await self._consume_async_iterable(result)
+
+        return result
 
     async def _wait_for_callbacks(
         self,
@@ -451,11 +512,15 @@ class ExperimentRunner:
         """Derive the final output from callbacks or observations."""
         for observation in reversed(observations):
             if observation.get("name") == "agent_final_response" and observation.get("output") is not None:
-                return observation.get("output")
+                val = observation.get("output")
+                if not ExperimentRunner._looks_like_generator_repr(val):
+                    return val
 
         for observation in reversed(observations):
             if observation.get("type") == "agent" and observation.get("output") is not None:
-                return observation.get("output")
+                val = observation.get("output")
+                if not ExperimentRunner._looks_like_generator_repr(val):
+                    return val
 
         send_messages = callback_messages.get("send") if callback_messages else None
         if send_messages:
@@ -464,6 +529,142 @@ class ExperimentRunner:
 
         return None
 
+    @staticmethod
+    def _looks_like_generator_repr(value: Any) -> bool:
+        if not isinstance(value, str):
+            return False
+        s = value.strip().lower()
+        return s.startswith("<async_generator ") or s.startswith("<generator ") or " object at 0x" in s
+
+    async def _consume_async_gen(self, func: Callable, kwargs: Dict[str, Any]) -> Any:
+        """Consume an async generator function by joining text chunks resolved from events."""
+        gen = func(**kwargs)
+        return await self._consume_async_iterable(gen)
+
+    async def _consume_async_iterable(self, agen: Any) -> Any:
+        """Consume async iterable items, extracting text via runner.stream_output_path.
+
+        Uses a copied contextvars context to ensure FluxLoop context is preserved
+        across async iteration boundaries created by upstream frameworks.
+        """
+        # Prefer ChatKit-like path by default; configurable via runner.stream_output_path
+        path = (getattr(self.config.runner, "stream_output_path", None) or "update.delta").split(".")
+        chunks: List[str] = []
+
+        ctx = contextvars.copy_context()
+
+        while True:
+            try:
+                item = await ctx.run(agen.__anext__)
+            except StopAsyncIteration:
+                break
+
+            val = self._get_by_path(item, path)
+            if isinstance(val, str) and val:
+                chunks.append(val)
+                continue
+
+            fallback = self._extract_stream_text(item)
+            if fallback:
+                chunks.append(fallback)
+        return "".join(chunks) if chunks else None
+
+    @staticmethod
+    def _get_by_path(obj: Any, parts: List[str]) -> Any:
+        cur: Any = obj
+        for key in parts:
+            if cur is None:
+                return None
+            if isinstance(cur, dict):
+                cur = cur.get(key)
+            else:
+                cur = getattr(cur, key, None)
+        return cur
+
+    @staticmethod
+    def _extract_stream_text(event: Any) -> Optional[str]:
+        """Best-effort extraction of text payloads from streaming events."""
+
+        update = getattr(event, "update", None)
+        if update is not None:
+            delta = getattr(update, "delta", None)
+            if isinstance(delta, str) and delta:
+                return delta
+
+            content = getattr(update, "content", None)
+            text = getattr(content, "text", None) if content is not None else None
+            if isinstance(text, str) and text:
+                return text
+
+        item = getattr(event, "item", None)
+        if item is not None:
+            content = getattr(item, "content", None)
+            if isinstance(content, list):
+                parts: List[str] = []
+                for piece in content:
+                    text = getattr(piece, "text", None)
+                    if isinstance(text, str) and text:
+                        parts.append(text)
+                if parts:
+                    return " ".join(parts)
+
+        text_attr = getattr(event, "text", None)
+        if isinstance(text_attr, str) and text_attr:
+            return text_attr
+        # Final deep fallback: search common fields recursively
+        return ExperimentRunner._deep_extract_text(event)
+
+    @staticmethod
+    def _deep_extract_text(obj: Any, *, _depth: int = 0) -> Optional[str]:
+        if _depth > 3 or obj is None:
+            return None
+        if isinstance(obj, str):
+            return obj if obj else None
+        # dict-like
+        if isinstance(obj, dict):
+            for key in ("delta", "text"):
+                val = obj.get(key)
+                if isinstance(val, str) and val:
+                    return val
+            # content as list of parts with text
+            content = obj.get("content")
+            if isinstance(content, list):
+                parts: List[str] = []
+                for piece in content:
+                    txt = ExperimentRunner._deep_extract_text(piece, _depth=_depth + 1)
+                    if txt:
+                        parts.append(txt)
+                if parts:
+                    return " ".join(parts)
+            # Recurse selected fields
+            for key in ("update", "message", "item", "data"):
+                val = obj.get(key)
+                txt = ExperimentRunner._deep_extract_text(val, _depth=_depth + 1)
+                if txt:
+                    return txt
+            return None
+        # object with attributes
+        for attr in ("delta", "text"):
+            val = getattr(obj, attr, None)
+            if isinstance(val, str) and val:
+                return val
+        for attr in ("content",):
+            val = getattr(obj, attr, None)
+            if isinstance(val, list):
+                parts: List[str] = []
+                for piece in val:
+                    txt = ExperimentRunner._deep_extract_text(piece, _depth=_depth + 1)
+                    if txt:
+                        parts.append(txt)
+                if parts:
+                    return " ".join(parts)
+        for attr in ("update", "message", "item", "data"):
+            val = getattr(obj, attr, None)
+            txt = ExperimentRunner._deep_extract_text(val, _depth=_depth + 1)
+            if txt:
+                return txt
+        return None
+
     @staticmethod
     def _extract_payload(args: Sequence[Any], kwargs: Dict[str, Any]) -> Any:
         if kwargs:
@@ -486,6 +687,33 @@ class ExperimentRunner:
             "args": list(args),
             "kwargs": kwargs,
         }
+
+    async def _coerce_result_to_text(self, value: Any) -> Optional[str]:
+        """Best-effort conversion of agent result into text for summaries.
+
+        Handles async generators/iterables, known streaming item shapes, and strings.
+        """
+        import inspect as _inspect
+
+        if value is None:
+            return None
+
+        if isinstance(value, str):
+            return value
+
+        # Async iterable/generator → consume to text
+        if _inspect.isasyncgen(value) or hasattr(value, "__aiter__"):
+            try:
+                return await self._consume_async_iterable(value)
+            except Exception:
+                return None
+
+        # Try extracting text from known event-like objects
+        fallback = self._extract_stream_text(value)
+        if fallback:
+            return fallback
+
+        return None
     
     def _save_results(self) -> None:
         """Save results to output directory."""
@@ -552,19 +780,43 @@ class ExperimentRunner:
         if not trace_ids:
             return
 
-        source_path = self.offline_dir / "observations.jsonl"
-        if not source_path.exists():
-            console.print(
-                f"[yellow]⚠️  Observations file not found: {source_path}[/yellow]"
-            )
+        # Gather candidate offline stores
+        candidates = []
+        try:
+            from fluxloop import get_config as _get_sdk_config  # type: ignore
+            sdk_dir = Path(_get_sdk_config().offline_store_dir)
+            candidates.append(sdk_dir / "observations.jsonl")
+        except Exception:
+            pass
+
+        candidates.append(self.offline_dir / "observations.jsonl")
+        # Legacy fallback removed now that SDK defaults to experiments/artifacts
+
+        # Unique existing paths in priority order
+        seen = set()
+        existing: list[Path] = []
+        for p in candidates:
+            try:
+                rp = p.resolve()
+            except Exception:
+                continue
+            if rp in seen:
+                continue
+            seen.add(rp)
+            if rp.exists():
+                existing.append(rp)
+
+        if not existing:
             return
 
         destination = self.output_dir / "observations.jsonl"
         copied = 0
+        seen_lines = set()
 
-        with source_path.open("r", encoding="utf-8") as src, destination.open(
-            "w", encoding="utf-8"
-        ) as dst:
+        with destination.open("w", encoding="utf-8") as dst:
+            for source_path in existing:
+                try:
+                    with source_path.open("r", encoding="utf-8") as src:
             for line in src:
                 if not line.strip():
                     continue
@@ -573,8 +825,14 @@ class ExperimentRunner:
                 except json.JSONDecodeError:
                     continue
                 if record.get("trace_id") in trace_ids:
+                                key = (record.get("id"), record.get("start_time"))
+                                if key in seen_lines:
+                                    continue
                     dst.write(json.dumps(record) + "\n")
+                                seen_lines.add(key)
                     copied += 1
+                except OSError:
+                    continue
 
         console.print(
             f"[green]✅ Saved {copied} observations to {destination.name}[/green]"
